#include <stdio.h>

#include <sys/file.h>

#include "IoState.h"
#include "IoObject.h"
#include "IoMessage.h"
#include "IoString.h"
#include "IoFile.h"

IoObject* MyIoObject_getSlot(IoState* state, IoObject* target, char* s) {
    IoString* str = IoState_stringWithCString_(state, s);
    return IoObject_getSlot_(target, str);
}

void MyIoObject_setSlot_to(IoState *state, IoObject *self,
                           char* s, IoObject *value) {
    IoString* str = IoState_stringWithCString_(state, s);
    IoObject_setSlot_to_(self, str, value);
}

void MyIoObject_addMethod(IoState *state, IoObject *self,
                          char* s, IoMethodFunc *fp) {
    IoString* str = IoState_stringWithCString_(state, s);
    IoObject_addMethod_(self, str, fp);
}

#define DATA(self) ((IoFileData *)self->data)

IoObject* IoFile_lockSH(IoObject* self, IoObject* locals, IoMessage* m) {
    flock(fileno(DATA(self)->stream), LOCK_SH);
}

IoObject* IoFile_lockEX(IoObject* self, IoObject* locals, IoMessage* m) {
    flock(fileno(DATA(self)->stream), LOCK_EX);
}

IoObject* IoFile_unlock(IoObject* self, IoObject* locals, IoMessage* m) {
    flock(fileno(DATA(self)->stream), LOCK_UN);
}

int main(int argc, const char* argv[]) {
    IoState* ioState;
    IoObject* lobby;
    IoObject* file;

    ioState = IoState_new();
    IoState_argc_argv_(ioState, argc, argv);

    IoState_pauseGarbageCollector(ioState);

    lobby = IoState_lobby(ioState);

    file = MyIoObject_getSlot(ioState, lobby, "File");
    MyIoObject_addMethod(ioState, file, "lockSH", IoFile_lockSH);
    MyIoObject_addMethod(ioState, file, "lockEX", IoFile_lockEX);
    MyIoObject_addMethod(ioState, file, "unlock", IoFile_unlock);
    MyIoObject_setSlot_to(ioState, lobby, "LFile", file);

    IoState_resumeGarbageCollector(ioState);
    IoState_clearRetainStack(ioState);

    IoState_doFile_(ioState, "syard.io");

    return 0;
}
